# Generated by Django 5.2 on 2025-05-02 14:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('careplans', '0005_initial'),
        ('claimresponses', '0005_initial'),
        ('coverages', '0002_initial'),
        ('devicedefinitions', '0001_initial'),
        ('immunizationrecommendations', '0003_immunizationrecommendationsupportingimmunizationreference_immunization_and_more'),
        ('medicationrequests', '0003_initial'),
        ('procedures', '0002_initial'),
        ('provenances', '0002_provenance_activity_provenance_authorization_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationrequestbasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_based_on_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='medicationrequestbasedonreference',
            name='immunization_recommendation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_based_on_immunization_recommendation', to='immunizationrecommendations.immunizationrecommendation'),
        ),
        migrations.AddField(
            model_name='medicationrequestinsurancereference',
            name='claim_response',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_insurance_reference_claim_response', to='claimresponses.claimresponse'),
        ),
        migrations.AddField(
            model_name='medicationrequestinsurancereference',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_insurance_reference_coverage', to='coverages.coverage'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='device_definition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_device_definition', to='devicedefinitions.devicedefinition'),
        ),
        migrations.AddField(
            model_name='medicationrequestprovinancereference',
            name='provenance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_provinance_reference_provenance', to='provenances.provenance'),
        ),
        migrations.AddField(
            model_name='medicationrequestreasonreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_reason_reference_procedure', to='procedures.procedure'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PatientContact',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('gender', models.CharField(blank=True, choices=[('male', 'male'), ('female', 'female'), ('other', 'other'), ('unknown', 'unknown')], max_length=20, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientGroupReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientLink',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(blank=True, choices=[('replaced_by', 'replaced-by'), ('replaces', 'replaces'), ('refer', 'refer'), ('seealso', 'seealso')], max_length=20, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientOrganizationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientPractitionerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PatientRelatedPersonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Patient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('birth_date', models.DateField(blank=True, null=True)),
                ('gender', models.CharField(blank=True, choices=[('male', 'male'), ('female', 'female'), ('other', 'other'), ('unknown', 'unknown')], max_length=20, null=True)),
                ('middle_name', models.CharField(blank=True, max_length=255, null=True)),
                ('email', models.EmailField(max_length=254, null=True, unique=True, verbose_name='email address')),
                ('deceased_boolean', models.BooleanField(default=True)),
                ('deceased_date_time', models.DateTimeField(blank=True, null=True)),
                ('active', models.BooleanField(default=True)),
                ('multiple_birth_boolean', models.BooleanField(default=False)),
                ('multiple_birth_integer', models.IntegerField(blank=True, null=True)),
                ('address', models.ManyToManyField(blank=True, related_name='patient_address', to='base.address')),
                ('communication', models.ManyToManyField(blank=True, related_name='patient_communication', to='base.communication')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Flag serializers."""

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer

from .models import (
    Flag,
    FlagAuthorReference,
    FlagSubjectReference,
    FlagSupportingInfoReference,
)


class FlagSupportingInfoReferenceSerializer(BaseReferenceModelSerializer):
    """FlagSupportingInfoReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = FlagSupportingInfoReference
        exclude = ["created_at", "updated_at"]


class FlagAuthorReferenceSerializer(BaseReferenceModelSerializer):
    """FlagAuthorReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = FlagAuthorReference
        exclude = ["created_at", "updated_at"]


class FlagSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """FlagSubjectReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = FlagSubjectReference
        exclude = ["created_at", "updated_at"]


class FlagSerializer(BaseWritableNestedModelSerializer):
    """Flag serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    subject = FlagSubjectReferenceSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    author = FlagAuthorReferenceSerializer(many=False, required=False)
    supporting_info = FlagSupportingInfoReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Flag
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('bodystructures', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImagingSelectionBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionImageRegion3D',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('region_type', models.CharField(choices=[('point', 'Point'), ('multipoint', 'Multipoint'), ('polyline', 'Polyline'), ('polygon', 'Polygon'), ('ellipse', 'Ellipse'), ('ellipsoid', 'Ellipsoid')], max_length=255, null=True)),
                ('coordinate', django.contrib.postgres.fields.ArrayField(base_field=models.FloatField(null=True), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionInstance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('uid', models.CharField(max_length=255, null=True)),
                ('number', models.IntegerField(null=True)),
                ('sop_class', models.CharField(max_length=255, null=True)),
                ('subset', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255, null=True), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionInstanceImagingRegion2D',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('region_type', models.CharField(choices=[('point', 'Point'), ('polyline', 'Polyline'), ('multipoint', 'Multipoint'), ('circle', 'Circle'), ('ellipse', 'Ellipse')], max_length=255, null=True)),
                ('coordinate', django.contrib.postgres.fields.ArrayField(base_field=models.FloatField(null=True), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionPerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionPerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelectionSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImagingSelection',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('available', 'Available'), ('entered-in-error', 'Entered in Error'), ('inactive', 'Inactive'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('issued', models.DateTimeField(null=True)),
                ('study_uid', models.CharField(max_length=255, null=True)),
                ('series_uid', models.CharField(max_length=255, null=True)),
                ('series_number', models.IntegerField(null=True)),
                ('frame_of_reference_uid', models.CharField(max_length=255, null=True)),
                ('body_site', models.ManyToManyField(blank=True, related_name='imaging_selection_bodysite', to='bodystructures.bodystructurecodeablereference')),
                ('category', models.ManyToManyField(blank=True, related_name='image_selection_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='image_selection_code', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""supply delivery app."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class SupplydeliveryConfig(AppConfig):
    """supply delivery app config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.supplydeliveries"
    verbose_name = _("Supply Delivery")



"""medication knowledges app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class MedicationknowledgesConfig(AppConfig):
    """Medication Knowledges app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.medicationknowledges"
    verbose_name = _("Medication Knowledges")



# Generated by Django 5.2 on 2025-05-02 14:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('base', '0005_initial'),
        ('chargeitems', '0004_initial'),
        ('invoices', '0003_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='CharItemReference',
            new_name='ChargeItemReference',
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('endpoints', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
                ('alias', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('website', models.URLField(null=True)),
                ('email', models.EmailField(max_length=254, null=True)),
                ('status', models.CharField(choices=[('pending', 'Pending'), ('active', 'Active'), ('inactive', 'Inactive')], default='pending', max_length=50)),
                ('active', models.BooleanField(default=True)),
                ('description', models.TextField(null=True)),
                ('contact', models.ManyToManyField(blank=True, related_name='organization_contact', to='base.extendedcontactdetail')),
                ('endpoint', models.ManyToManyField(blank=True, related_name='organization_endpoint', to='endpoints.endpointreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='organization_identifier', to='base.identifier')),
                ('part_of', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_part_of', to='base.organizationreference')),
                ('qualification', models.ManyToManyField(blank=True, related_name='organization_qualification', to='base.qualification')),
                ('type', models.ManyToManyField(blank=True, related_name='organization_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrganizationCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_codeable_concept_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_codeable_concept_reference', to='base.organizationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Slot serializer module."""

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
)
from dfhir.healthcareservices.serializers import (
    HealthCareServiceCodeableReferenceSerializer,
)

from .models import Slot, SlotReference


class SlotReferenceSerializer(BaseReferenceModelSerializer):
    """Slot reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = SlotReference
        exclude = ["created_at", "updated_at"]


class SlotSerializer(BaseWritableNestedModelSerializer):
    """Slot serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    service_type = HealthCareServiceCodeableReferenceSerializer(
        many=True, required=False
    )
    specialty = CodeableConceptSerializer(many=True, required=False)
    service_category = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Slot
        exclude = ["created_at", "updated_at"]



"""transport serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.coverages.serializers import CoverageClaimResponseReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer
from dfhir.transports.models import (
    Transport,
    TransportBasedOnReference,
    TransportInput,
    TransportOutput,
    TransportOwnerReference,
    TransportReference,
    TransportRequesterReference,
    TransportRestriction,
    TransportRestrictionRecipientReference,
)


class TransportReferenceSerializer(BaseReferenceModelSerializer):
    """Transport Reference Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = TransportReference
        exclude = ["created_at", "updated_at"]


class TransportBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Transport Based On Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = TransportBasedOnReference
        exclude = ["created_at", "updated_at"]


class TransportOwnerReferenceSerializer(BaseReferenceModelSerializer):
    """TransportOwnerReferenceSerializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = TransportOwnerReference
        exclude = ["created_at", "updated_at"]


class TransportRestrictionRecipientReferenceSerializer(BaseReferenceModelSerializer):
    """Transport Restriction Recipient Reference Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = TransportRestrictionRecipientReference
        exclude = ["created_at", "updated_at"]


class TransportRestrictionSerializer(WritableNestedModelSerializer):
    """TransportRestrictionSerializer."""

    period = PeriodSerializer(required=False)
    recipient = TransportRestrictionRecipientReferenceSerializer(
        required=False, many=True
    )

    class Meta:
        """Meta."""

        model = TransportRestriction
        exclude = ["created_at", "updated_at"]


class TransportInputSerializer(WritableNestedModelSerializer):
    """TransportInputSerializer."""

    type = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta."""

        model = TransportInput
        exclude = ["created_at", "updated_at"]


class TransportOutputSerializer(WritableNestedModelSerializer):
    """TransportOutputSerializer."""

    type = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta."""

        model = TransportOutput
        exclude = ["created_at", "updated_at"]


class TransportRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """TransportRequesterReferenceSerializer."""

    identifier = IdentifierSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = TransportRequesterReference
        exclude = ["created_at", "updated_at"]


class TransportSerializer(BaseWritableNestedModelSerializer):
    """transport serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = TransportBasedOnReferenceSerializer(required=False, many=True)
    group_identifier = IdentifierSerializer(required=False)
    part_of = TransportReferenceSerializer(required=False, many=True)
    status_reason = CodeableConceptSerializer(required=False)
    code = CodeableConceptSerializer(required=False)
    focus = ReferenceSerializer(required=False)
    for_value = ReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    requester = TransportRequesterReferenceSerializer(required=False)
    performer_type = CodeableConceptSerializer(required=False, many=True)
    owner = TransportOwnerReferenceSerializer(required=False)
    location = LocationReferenceSerializer(required=False)
    insurance = CoverageClaimResponseReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    relevant_history = ProvenanceReferenceSerializer(many=True, required=False)
    restriction = TransportRestrictionSerializer(required=False)
    input = TransportInputSerializer(many=True, required=False)
    output = TransportOutputSerializer(many=True, required=False)
    requested_location = LocationReferenceSerializer(required=False)
    current_location = LocationReferenceSerializer(required=False)
    reason = CodeableConceptSerializer(required=False)
    history = TransportReferenceSerializer(required=False)

    class Meta:
        """Meta."""

        model = Transport
        exclude = ["created_at", "updated_at"]
        rename_fields = {
            "for_value": "for",
        }



"""Slots filters module."""

from django_filters import DateTimeFromToRangeFilter
from django_filters import rest_framework as filters

from .models import Slot


class SlotsFilter(filters.FilterSet):
    """Slots filter."""

    start_date_time = DateTimeFromToRangeFilter(field_name="start_date_time")
    end_date_time = DateTimeFromToRangeFilter(field_name="end_date_time")
    status = filters.CharFilter(field_name="status")
    practitioner = filters.CharFilter(field_name="practitioner")

    class Meta:
        """Meta class."""

        model = Slot
        fields = ["start_date_time", "end_date_time", "status", "practitioner"]



"""inventory report urls."""

from django.urls import path

from dfhir.inventoryreports import views

app_name = "inventoryreports"

urlpatterns = [
    path(
        "inventoryreports/",
        views.InventoryReportListView.as_view(),
        name="inventoryreport-list",
    ),
    path(
        "inventoryreports/<int:pk>/",
        views.InventoryReportDetailView.as_view(),
        name="inventoryreport-list",
    ),
]



"""Claims choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ClaimStatus(models.TextChoices):
    """Claim status choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class ClaimUseChoices(models.TextChoices):
    """Claim use choices."""

    CLAIM = "claim", _("Claim")
    PREAUTHORIZATION = "preauthorization", _("Preauthorization")
    PREDETERMINATION = "predetermination", _("Predetermination")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0002_initial'),
        ('base', '0003_initial'),
        ('conditions', '0001_initial'),
        ('coverages', '0001_initial'),
        ('encounters', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='covers',
            field=models.ManyToManyField(blank=True, related_name='accounts_covers', to='encounters.encounterepisodeofcarereference'),
        ),
        migrations.AddField(
            model_name='account',
            name='currency',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_currency', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='account',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='accounts_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='account',
            name='owner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_owner', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='account',
            name='service_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_service_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='account',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='accounts_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='accountbalance',
            name='aggregate',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_balance_aggregate', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='accountbalance',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_balance_amount', to='base.money'),
        ),
        migrations.AddField(
            model_name='accountbalance',
            name='term',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_balance_term', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='account',
            name='balance',
            field=models.ManyToManyField(blank=True, related_name='accounts_balance', to='accounts.accountbalance'),
        ),
        migrations.AddField(
            model_name='accountcoverage',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_coverage_coverage', to='coverages.coverage'),
        ),
        migrations.AddField(
            model_name='account',
            name='coverage',
            field=models.ManyToManyField(blank=True, related_name='accounts_coverage', to='accounts.accountcoverage'),
        ),
        migrations.AddField(
            model_name='accountdiagnosis',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_diagnosis_condition', to='conditions.conditioncodeablereference'),
        ),
        migrations.AddField(
            model_name='accountdiagnosis',
            name='package_code',
            field=models.ManyToManyField(blank=True, related_name='account_diagnosis_package_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='accountdiagnosis',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='account_diagnosis_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='account',
            name='diagnosis',
            field=models.ManyToManyField(blank=True, related_name='accounts_diagnosis', to='accounts.accountdiagnosis'),
        ),
        migrations.AddField(
            model_name='accountguarantor',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='account_guarantor_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='account',
            name='guarantor',
            field=models.ManyToManyField(blank=True, related_name='accounts_guarantor', to='accounts.accountguarantor'),
        ),
    ]



"""Device choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceStatus(models.TextChoices):
    """Device status choices."""

    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('goals', '0002_initial'),
        ('medicationrequests', '0001_initial'),
        ('medicationstatements', '0002_initial'),
        ('nutritionintakes', '0002_initial'),
        ('nutritionorders', '0001_initial'),
        ('observations', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='goalacceptanceparticipantreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='goalacceptanceparticipantreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='goalacceptanceparticipantreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='goalacceptance',
            name='participant',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='goal_acceptance_participant', to='goals.goalacceptanceparticipantreference'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='medication_statement',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_medication_statement', to='medicationstatements.medicationstatement'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='nutrition_intake',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_nutrition_intake', to='nutritionintakes.nutritionintake'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
        migrations.AddField(
            model_name='goaladdressesreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='goal_addresses_reference_observation', to='observations.observation'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('activitydefinitions', '0002_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='activitydefinitionparticipanttypereference',
            name='biologically_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_definition_biologically_derived_product_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClinicalImpressionFinding',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('basis', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClinicalImpressionFindingItemCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClinicalImpressionFindingItemReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClinicalImpressionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ClinicalImpression',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('preparation', 'Preparation'), ('in-progress', 'In Progress'), ('not-done', 'Not Done'), ('on-hold', 'On Hold'), ('stopped', 'Stopped'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('effective_date_time', models.DateTimeField(null=True)),
                ('date', models.DateTimeField(null=True)),
                ('protocol', django.contrib.postgres.fields.ArrayField(base_field=models.URLField(), null=True, size=None)),
                ('summary', models.TextField(null=True)),
                ('change_pattern', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_change_pattern', to='base.codeableconcept')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clinical_impression_effective_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Explanation of Benefits views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import ExplanationOfBenefit
from .serializers import ExplanationOfBenefitSerializer


class ExplanationOfBenefitListView(APIView):
    """Explanation of Benefits list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ExplanationOfBenefitSerializer(many=True)})
    def get(self, request):
        """Get a list of explanation of benefits."""
        explanation_of_benefits = ExplanationOfBenefit.objects.all()
        serializer = ExplanationOfBenefitSerializer(explanation_of_benefits, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ExplanationOfBenefitSerializer,
        responses={201: ExplanationOfBenefitSerializer},
    )
    def post(self, request):
        """Create a explanation of benefits."""
        serializer = ExplanationOfBenefitSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ExplanationOfBenefitDetailView(APIView):
    """Explanation of benefit detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a care plan object."""
        try:
            return ExplanationOfBenefit.objects.get(pk=pk)
        except ExplanationOfBenefit.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ExplanationOfBenefitSerializer})
    def get(self, request, pk=None):
        """Get a care plan."""
        explanation_of_benefit = self.get_object(pk)
        serializer = ExplanationOfBenefitSerializer(explanation_of_benefit)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ExplanationOfBenefitSerializer,
        responses={200: ExplanationOfBenefitSerializer},
    )
    def patch(self, request, pk=None):
        """Update a care plan."""
        explanation_of_benefit = self.get_object(pk)
        serializer = ExplanationOfBenefitSerializer(
            explanation_of_benefit, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a care plan."""
        explanation_of_benefit = self.get_object(pk)
        explanation_of_benefit.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Family member history models."""

from django.db import models

from dfhir.base.models import (
    Age,
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    Range,
    TimeStampedModel,
)

from . import choices


class FamilyMemberHistoryParticipantActorReference(BaseReference):
    """Family Member History Participant Actor Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_related_person",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_device",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_organization",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor_reference_care_team",
        null=True,
    )


class FamilyMemberHistoryParticipant(TimeStampedModel):
    """Family Member History Participant model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_function",
        null=True,
    )
    actor = models.ForeignKey(
        FamilyMemberHistoryParticipantActorReference,
        on_delete=models.CASCADE,
        related_name="family_member_history_participant_actor",
    )


class FamilyMemberHistoryReasonReference(BaseReference):
    """Family Member History Reason Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_identifier",
        null=True,
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_condition",
        null=True,
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_observation",
        null=True,
    )
    allergy_intolerance = models.ForeignKey(
        "allergyintolerances.AllergyIntolerance",
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_allergy_intolerance",
        null=True,
    )
    # questionnaire_response = models.ForeignKey(
    #     "questionnaireresponses.QuestionnaireResponse",
    #     on_delete=models.CASCADE,
    #     related_name="family_member_history_reason_reference_questionnaire_response",
    #     null=True,
    # )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_diagnostic_report",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_reference_document_reference",
        null=True,
    )


class FamilyMemberHistoryReasonCodeableReference(TimeStampedModel):
    """Family Member History Reason Codeable Reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_codeable_reference_codeable_concept",
        null=True,
    )
    reference = models.ForeignKey(
        FamilyMemberHistoryReasonReference,
        on_delete=models.CASCADE,
        related_name="family_member_history_reason_codeable_reference_reference",
        null=True,
    )


class FamilyMemberHistoryCondition(TimeStampedModel):
    """Family Member History Condition model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_condition_code",
        null=True,
    )
    outcome = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_condition_outcome",
        null=True,
    )
    contributed_to_death = models.BooleanField(null=True)
    onset_age = models.ForeignKey(
        Age,
        on_delete=models.CASCADE,
        related_name="family_member_history_condition_onset_age",
        null=True,
    )
    onset_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        related_name="family_member_history_condition_onset_range",
        null=True,
    )
    onset_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="family_member_history_condition_onset_period",
        null=True,
    )
    onset_string = models.CharField(max_length=255, null=True)
    note = models.ManyToManyField(
        Annotation, related_name="family_member_history_condition_note", blank=True
    )


class FamilyMemberHistoryProcedure(TimeStampedModel):
    """Family Member History Procedure model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_procedure_code",
        null=True,
    )
    outcome = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="family_member_history_procedure_outcome",
        null=True,
    )
    contributed_to_death = models.BooleanField(null=True)
    performed_age = models.ForeignKey(
        Age,
        on_delete=models.CASCADE,
        related_name="family_member_history_procedure_performed_age",
        null=True,
    )
    performed_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        related_name="family_member_history_procedure_performed_range",
        null=True,
    )
    performed_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="family_member_history_procedure_performed_period",
        null=True,
    )
    performed_string = models.CharField(max_length=255, null=True)
    performed_date_time = models.DateTimeField(null=True)
    note = models.ManyToManyField(
        Annotation, related_name="family_member_history_procedure_note", blank=True
    )


class FamilyMemberHistory(TimeStampedModel):
    """Family Member History model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="family_member_history_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.FamilyMemberHistoryStatus.choices
    )
    data_absent_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="family_member_history_data_absent_reason",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="family_member_history_patient",
    )
    date = models.DateTimeField(null=True)
    participant = models.ManyToManyField(
        FamilyMemberHistoryParticipant,
        related_name="family_member_history_participant",
        blank=True,
    )
    name = models.CharField(max_length=255, null=True)
    relationship = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_relationship",
    )
    sex = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_sex",
    )
    born_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_born_period",
    )
    born_date = models.DateTimeField(null=True)
    born_string = models.CharField(max_length=255, null=True)
    age_age = models.ForeignKey(
        Age,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_age_age",
    )
    age_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_age_range",
    )
    age_string = models.CharField(max_length=255, null=True)
    estimated_age = models.BooleanField(null=True)
    deceased_boolean = models.BooleanField(null=True)
    deceased_age = models.ForeignKey(
        Age,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_deceased_age",
    )
    deceased_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        null=True,
        related_name="family_member_history_deceased_range",
    )
    deceased_date = models.DateTimeField(null=True)
    deceased_string = models.CharField(max_length=255, null=True)
    reason = models.ManyToManyField(
        FamilyMemberHistoryReasonCodeableReference,
        related_name="family_member_history_reason",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation, related_name="family_member_history_note", blank=True
    )
    condition = models.ManyToManyField(
        FamilyMemberHistoryCondition,
        related_name="family_member_history_condition",
        blank=True,
    )
    procedure = models.ManyToManyField(
        FamilyMemberHistoryProcedure,
        related_name="family_member_history_procedure",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('devices', '0001_initial'),
        ('inventoryitems', '0001_initial'),
        ('locations', '0001_initial'),
        ('medications', '0001_initial'),
        ('nutritionproducts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='InventoryReportInventoryListing',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('counting_date_time', models.DateTimeField(null=True)),
                ('item_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_status', to='base.codeableconcept')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing', to='locations.locationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryReport',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('requested', 'Requested'), ('active', 'Active'), ('entered-in-error', 'Entered In Error')], max_length=255, null=True)),
                ('count_type', models.CharField(choices=[('snapshot', 'Snapshot'), ('difference', 'Difference')], max_length=255, null=True)),
                ('reported_date_time', models.DateTimeField(null=True)),
                ('identifier', models.ManyToManyField(blank=True, related_name='inventory_report', to='base.identifier')),
                ('note', models.ManyToManyField(blank=True, related_name='inventory_report', to='base.annotation')),
                ('operation_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_operational_type', to='base.codeableconcept')),
                ('operation_type_reason', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_operational_reason', to='base.codeableconcept')),
                ('reporting_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report', to='base.period')),
                ('inventory_listing', models.ManyToManyField(blank=True, related_name='inventory_report', to='inventoryreports.inventoryreportinventorylisting')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryReportInventoryListingItemItemCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_item_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='inventoryreportinventorylisting',
            name='item',
            field=models.ManyToManyField(blank=True, related_name='inventory_report_inventory_listing', to='inventoryreports.inventoryreportinventorylistingitemitemcodeablereference'),
        ),
        migrations.CreateModel(
            name='InventoryReportInventoryListingItemItemReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('biologically_derived_product', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_reference_identifier', to='base.identifier')),
                ('inventory_item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_inventory_item', to='inventoryitems.inventoryitem')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_medication', to='medications.medication')),
                ('nutrition_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_nutrition_product', to='nutritionproducts.nutritionproduct')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='inventoryreportinventorylistingitemitemcodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_item_codeable_reference_reference', to='inventoryreports.inventoryreportinventorylistingitemitemreference'),
        ),
        migrations.CreateModel(
            name='InventoryReportInventoryListingItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('category', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_category', to='base.codeableconcept')),
                ('quality', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_inventory_listing_item_quality', to='base.quantity')),
                ('item', models.ManyToManyField(blank=True, related_name='inventory_report_inventory_listing_item', to='inventoryreports.inventoryreportinventorylistingitemitemreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryReportReporterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_reporter_reference_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='inventory_report_reporter_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Enrollment Requests Models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    Identifier,
    OrganizationReference,
    TimeStampedModel,
)

from . import choices


class EnrollmentRequestProviderReference(BaseReference):
    """Enrollment Request Provider Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="enrollment_request_provider_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="enrollment_request_provider_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="enrollment_request_provider_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="enrollment_request_provider_reference_organization",
        null=True,
    )


class EnrollmentRequest(TimeStampedModel):
    """Enrollment request model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="enrollment_request_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.EnrollmentRequestStatus.choices,
        null=True,
    )
    created = models.DateTimeField(null=True)
    insurer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="enrollment_request_insurer",
        null=True,
    )
    provider = models.ForeignKey(
        EnrollmentRequestProviderReference,
        on_delete=models.CASCADE,
        related_name="enrollment_request_provider",
        null=True,
    )
    candidate = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="enrollment_request_candidate",
        null=True,
    )
    coverage = models.ForeignKey(
        "coverages.CoverageReference",
        on_delete=models.CASCADE,
        related_name="enrollment_request_coverage",
        null=True,
    )


class EnrollmentRequestsReference(BaseReference):
    """Enrollment Requests Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="enrollment_requests_reference_identifier",
        null=True,
    )
    enrollment_request = models.ForeignKey(
        EnrollmentRequest,
        on_delete=models.CASCADE,
        related_name="enrollment_requests_reference_enrollment_request",
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('episodeofcares', '0002_initial'),
        ('healthcareservices', '0001_initial'),
        ('observations', '0002_initial'),
        ('patients', '0002_initial'),
        ('procedures', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='episodeofcare',
            name='referral_request',
            field=models.ManyToManyField(blank=True, related_name='episode_of_care_referral_request', to='servicerequests.servicerequestreference'),
        ),
        migrations.AddField(
            model_name='episodeofcare',
            name='subject',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_patient', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='episodeofcare',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='episode_of_care_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='episodeofcarediagnosis',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_diagnosis_condition', to='conditions.conditioncodeablereference'),
        ),
        migrations.AddField(
            model_name='episodeofcarediagnosis',
            name='use',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_diagnosis_use', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='episodeofcare',
            name='diagnosis',
            field=models.ManyToManyField(blank=True, related_name='episode_of_care_diagnosis', to='episodeofcares.episodeofcarediagnosis'),
        ),
        migrations.AddField(
            model_name='episodeofcarereason',
            name='use',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_use', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='episodeofcarereason',
            name='value',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value', to='episodeofcares.episodeofcarereasonvaluecodeablereference'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluereference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluereference',
            name='healthcare_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_reference_healthcare_service', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluereference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluereference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='episodeofcarereasonvaluecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reason_value_codeable_reference_reference', to='episodeofcares.episodeofcarereasonvaluereference'),
        ),
        migrations.AddField(
            model_name='episodeofcarereference',
            name='episode_of_care',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reference_episode_of_care', to='episodeofcares.episodeofcare'),
        ),
        migrations.AddField(
            model_name='episodeofcarereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='episodeofcarestatushistory',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='episode_of_care_status_history_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='episodeofcare',
            name='status_history',
            field=models.ManyToManyField(blank=True, related_name='episode_of_care_status_history', to='episodeofcares.episodeofcarestatushistory'),
        ),
    ]



"""Explanation of Benefits app configuration."""

from django.apps import AppConfig


class ExplanationofbenefitsConfig(AppConfig):
    """Explanation of Benefits app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.explanationofbenefits"
    verbose_name = "Explanation of Benefits"



"""Coverage Eligibility Responses views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import CoverageEligibilityResponse
from .serializers import CoverageEligibilityResponseSerializer


class CoverageEligibilityResponseListView(APIView):
    """Coverage Eligibility Response list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: CoverageEligibilityResponseSerializer(many=True)})
    def get(self, request):
        """Get all coverage eligibility responses."""
        coverage_eligibility_responses = CoverageEligibilityResponse.objects.all()
        serializer = CoverageEligibilityResponseSerializer(
            coverage_eligibility_responses, many=True
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=CoverageEligibilityResponseSerializer,
        responses={201: CoverageEligibilityResponseSerializer},
    )
    def post(self, request):
        """Create a coverage eligibility response."""
        serializer = CoverageEligibilityResponseSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class CoverageEligibilityResponseDetailView(APIView):
    """Coverage Eligibility Response detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get coverage eligibility response object."""
        try:
            return CoverageEligibilityResponse.objects.get(id=pk)
        except CoverageEligibilityResponse.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: CoverageEligibilityResponseSerializer})
    def get(self, request, pk=None):
        """Get coverage eligibility response detail."""
        coverage_eligibility_response = self.get_object(pk)
        serializer = CoverageEligibilityResponseSerializer(
            coverage_eligibility_response
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=CoverageEligibilityResponseSerializer,
        responses={200: CoverageEligibilityResponseSerializer},
    )
    def patch(self, request, pk=None):
        """Update coverage eligibility response."""
        coverage_eligibility_response = self.get_object(pk)
        serializer = CoverageEligibilityResponseSerializer(
            coverage_eligibility_response, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete coverage eligibility response."""
        coverage_eligibility_response = self.get_object(pk)
        coverage_eligibility_response.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('devicedefinitions', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ObservationDefinitionQualifiedValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('gender', models.CharField(choices=[('male', 'Male'), ('female', 'Female'), ('other', 'Other'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('condition', models.CharField(max_length=255, null=True)),
                ('range_category', models.CharField(choices=[('reference', 'Reference'), ('critical', 'Critical'), ('absolute', 'Absolute')], max_length=255, null=True)),
                ('age', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_qualified_value_age', to='base.range')),
                ('applies_to', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_qualified_value_applies_to', to='base.codeableconcept')),
                ('context', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_qualified_value_context', to='base.codeableconcept')),
                ('gestational_age', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_qualified_value_gestational_age', to='base.range')),
                ('interpretation', models.ManyToManyField(blank=True, related_name='observation_definition_qualified_value_interpretation', to='base.codeableconcept')),
                ('range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_qualified_value_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationDefinitionComponent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('permitted_data_type', models.CharField(choices=[('Quantity', 'Quantity'), ('codeableconcept', 'Codeable Concept'), ('string', 'String'), ('boolean', 'Boolean'), ('integer', 'Integer'), ('range', 'Range'), ('ratio', 'Ratio'), ('sampled_data', 'Sampled Data'), ('time', 'Time'), ('date', 'Date'), ('datetime', 'Datetime'), ('period', 'Period')], max_length=255, null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.codeableconcept')),
                ('permitted_unit', models.ManyToManyField(blank=True, related_name='observation_definition_component_permitted_unit', to='base.coding')),
                ('qulaified_value', models.ManyToManyField(blank=True, related_name='observation_definition_component_qualified_value', to='observationdefinitions.observationdefinitionqualifiedvalue')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('url', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('version_algorithm_string', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('draft', 'Draft'), ('active', 'Active'), ('retired', 'Retired'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('experimental', models.BooleanField(default=False)),
                ('date', models.DateTimeField(null=True)),
                ('publisher', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('purpose', models.TextField(null=True)),
                ('copyright', models.TextField(null=True)),
                ('copyright_label', models.CharField(max_length=255, null=True)),
                ('approval_date', models.DateTimeField(null=True)),
                ('last_review_date', models.DateTimeField(null=True)),
                ('derived_from_uri', models.URLField(null=True)),
                ('permitted_data_type', models.CharField(choices=[('Quantity', 'Quantity'), ('codeableconcept', 'Codeable Concept'), ('string', 'String'), ('boolean', 'Boolean'), ('integer', 'Integer'), ('range', 'Range'), ('ratio', 'Ratio'), ('sampled_data', 'Sampled Data'), ('time', 'Time'), ('date', 'Date'), ('datetime', 'Datetime'), ('period', 'Period')], max_length=255, null=True)),
                ('multiple_results_allowed', models.BooleanField(default=False)),
                ('preferred_report_name', models.CharField(max_length=255, null=True)),
                ('body_site', models.ManyToManyField(blank=True, related_name='observation_definition_body_site', to='base.codeableconcept')),
                ('category', models.ManyToManyField(blank=True, related_name='observation_definition_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_code', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='observation_definition_contact', to='base.contactdetail')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_device', to='devicedefinitions.devicedefinition')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_effective_period', to='base.period')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_identifier', to='base.identifier')),
                ('jurisdiction', models.ManyToManyField(blank=True, related_name='observation_definition_jurisdiction', to='base.codeableconcept')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_method', to='base.codeableconcept')),
                ('performer_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_performer_type', to='base.codeableconcept')),
                ('permitted_unit', models.ManyToManyField(blank=True, related_name='observation_definition_permitted_unit', to='base.coding')),
                ('subject', models.ManyToManyField(blank=True, related_name='observation_definition_subject', to='base.codeableconcept')),
                ('use_context', models.ManyToManyField(blank=True, related_name='observation_definition_use_context', to='base.usagecontext')),
                ('version_algorithm_coding', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_version_algorithm_coding', to='base.coding')),
                ('component', models.ManyToManyField(blank=True, related_name='observation_definition_component', to='observationdefinitions.observationdefinitioncomponent')),
                ('qualified_value', models.ManyToManyField(blank=True, related_name='observation_definition_qualified_value', to='observationdefinitions.observationdefinitionqualifiedvalue')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ObservationDefinitionQuestionnaireReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_questionnaire_reference_identifier', to='base.identifier')),
                ('observatoin_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='observationdefinitions.observationdefinition')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='observationdefinition',
            name='has_member',
            field=models.ManyToManyField(blank=True, related_name='observation_definition_has_member', to='observationdefinitions.observationdefinitionquestionnairereference'),
        ),
        migrations.CreateModel(
            name='ObservationDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_reference_identifier', to='base.identifier')),
                ('observation_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_definition_reference_observation_definition', to='observationdefinitions.observationdefinition')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""nutrition intake views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.nutritionintakes.models import NutritionIntake
from dfhir.nutritionintakes.serializers import NutritionIntakeSerializer


class NutritionIntakeListView(APIView):
    """nutrition intake list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: NutritionIntakeSerializer(many=True)})
    def get(self, request):
        """Get all nutrition intakes."""
        nutrition_intakes = NutritionIntake.objects.all()
        serializer = NutritionIntakeSerializer(nutrition_intakes, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=NutritionIntakeSerializer, responses={201: NutritionIntakeSerializer}
    )
    def post(self, request):
        """Create a new nutrition intake."""
        serializer = NutritionIntakeSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class NutritionIntakeDetailView(APIView):
    """nutrition intake detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get nutrition intake object."""
        try:
            return NutritionIntake.objects.get(pk=pk)
        except NutritionIntake.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: NutritionIntakeSerializer})
    def get(self, request, pk=None):
        """Get a nutrition intake."""
        nutrition_intake = self.get_object(pk)
        serializer = NutritionIntakeSerializer(nutrition_intake)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=NutritionIntakeSerializer, responses={200: NutritionIntakeSerializer}
    )
    def patch(self, request, pk=None):
        """Update a nutrition intake."""
        nutrition_intake = self.get_object(pk)
        serializer = NutritionIntakeSerializer(
            nutrition_intake, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a nutrition intake."""
        nutrition_intake = self.get_object(pk)
        nutrition_intake.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Detected issues views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import DetectedIssue
from .serializers import DetectedIssueSerializer


class DetectedIssueListView(APIView):
    """Detected issue list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: DetectedIssueSerializer(many=True)})
    def get(self, request):
        """Get a list of detected issues."""
        detected_issues = DetectedIssue.objects.all()
        serializer = DetectedIssueSerializer(detected_issues, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=DetectedIssueSerializer, responses={201: DetectedIssueSerializer}
    )
    def post(self, request):
        """Create a detected issue."""
        serializer = DetectedIssueSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_201_CREATED)


class DetectedIssueDetailView(APIView):
    """Detected issue detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a detected issue object."""
        try:
            return DetectedIssue.objects.get(pk=pk)
        except DetectedIssue.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: DetectedIssueSerializer})
    def get(self, request, pk):
        """Get a detected issue."""
        detected_issue = self.get_object(pk)
        serializer = DetectedIssueSerializer(detected_issue)
        return Response(serializer.data)

    @extend_schema(
        request=DetectedIssueSerializer, responses={200: DetectedIssueSerializer}
    )
    def patch(self, request, pk):
        """Update a detected issue."""
        detected_issue = self.get_object(pk)
        serializer = DetectedIssueSerializer(detected_issue, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a detected issue."""
        detected_issue = self.get_object(pk)
        detected_issue.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Family member history views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import FamilyMemberHistory
from .serializers import FamilyMemberHistorySerializer


class FamilyMemberHistoryListView(APIView):
    """Family member history list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: FamilyMemberHistorySerializer(many=True)})
    def get(self, request):
        """Get a list of family member histories."""
        family_member_histories = FamilyMemberHistory.objects.all()
        serializer = FamilyMemberHistorySerializer(family_member_histories, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=FamilyMemberHistorySerializer,
        responses={201: FamilyMemberHistorySerializer},
    )
    def post(self, request):
        """Create a family member history."""
        serializer = FamilyMemberHistorySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_201_CREATED)


class FamilyMemberHistoryDetailView(APIView):
    """Family member history detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a family member history object."""
        try:
            return FamilyMemberHistory.objects.get(pk=pk)
        except FamilyMemberHistory.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: FamilyMemberHistorySerializer})
    def get(self, request, pk):
        """Get a family member history."""
        family_member_history = self.get_object(pk)
        serializer = FamilyMemberHistorySerializer(family_member_history)
        return Response(serializer.data)

    @extend_schema(
        request=FamilyMemberHistorySerializer,
        responses={200: FamilyMemberHistorySerializer},
    )
    def patch(self, request, pk):
        """Update a family member history."""
        family_member_history = self.get_object(pk)
        serializer = FamilyMemberHistorySerializer(
            family_member_history, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a family member history."""
        family_member_history = self.get_object(pk)
        family_member_history.delete()

        return Response(status=status.HTTP_204_NO_CONTENT)



"""Models for service requests."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.patients.models import Patient
from dfhir.practitioners.models import Practitioner

from .choices import (
    ServiceRequestIntent,
    ServiceRequestPriority,
    ServiceRequestStatus,
)


class Parameter(TimeStampedModel):
    """parameter model."""

    display = models.CharField(max_length=255)
    description = models.CharField(max_length=255)


class Reference(TimeStampedModel):
    """reference model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class ServiceRequestCategory(TimeStampedModel):
    """service request category model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class AsNeeded(TimeStampedModel):
    """as needed model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class ProcedureReason(TimeStampedModel):
    """reason model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class BodySite(TimeStampedModel):
    """body site model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class ProcedureCodes(TimeStampedModel):
    """procedure codes model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class ServiceRequestBasedOnReference(BaseReference):
    """service request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_based_on_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.SET_NULL,
        related_name="service_request_based_on_care_paln",
        null=True,
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_based_on_service_request",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_based_on_medication_request",
    )
    # TODO: request_orchestration = models.ForeignKey("RequestOrchestration", on_delete=models.SET_NULL, null=True)
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_based_on_nutrition_order",
    )


class OrderDetailParameterFocusReference(BaseReference):
    """order detail parameter focus reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_identifier",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_device",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_device_definition",
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_device_request",
    )
    supply_request = models.ForeignKey(
        "supplyrequests.SupplyRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_supply_request",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_medication",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_medication_request",
    )
    biological_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_reference_biological_derived_product",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_substance",
    )


class OrderDetailParameterFocusCodeableReference(TimeStampedModel):
    """order detail parameter focus codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        OrderDetailParameterFocusReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="order_detail_parameter_focus_codeable_reference_reference",
    )


class OrderDetailParameter(TimeStampedModel):
    """order detail parameter."""

    code = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_code",
    )
    value_quantity = models.ForeignKey(
        "base.Quantity",
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_value_quantity",
    )
    value_ratio = models.ForeignKey(
        "base.Ratio",
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_value_ratio",
    )
    value_range = models.ForeignKey(
        "base.Range",
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_value_range",
    )
    value_boolean = models.BooleanField(default=False)
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_value_codeable_concept",
    )
    value_string = models.CharField(max_length=255, null=True)
    value_period = models.ForeignKey(
        "base.Period",
        null=True,
        on_delete=models.CASCADE,
        related_name="order_detail_parameter_value_period",
    )


class ServiceRequestOrderDetail(TimeStampedModel):
    """service request order detail model."""

    parameter_focus = models.ForeignKey(
        OrderDetailParameterFocusCodeableReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_order_detail_parameter_focus",
    )
    parameter = models.ManyToManyField(
        OrderDetailParameter,
        related_name="service_request_order_detail_parameter",
        blank=True,
    )


class ServiceRequestPatientInstruction(TimeStampedModel):
    """patient instruction model."""

    instruction_markdown = models.TextField(null=True)
    instruction_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="patient_instruction_document_reference",
    )


class ServiceRequestSubjectReference(BaseReference):
    """service request subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject_group",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject_location",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject_device",
    )


class ServiceRequestRequesterReference(BaseReference):
    """service request requester reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_identifier",
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_organization",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_related_person",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester_device",
    )


class ServiceRequestPerformerReference(BaseReference):
    """service request performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_care_team",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_healthcare_service",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_patient",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_device",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_performer_related_person",
    )


class ServiceRequestReasonReference(BaseReference):
    """service request reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_diagnostic_report",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_document_reference",
    )
    detected_issue = models.ForeignKey(
        "detectedissues.DetectedIssue",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_detected_issue",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_procedure",
    )


class ServiceRequestReasonCodeableReference(TimeStampedModel):
    """service request codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        ServiceRequestReasonReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reason_codeable_reference_reference",
    )


class ServiceRequestReference(BaseReference):
    """service request reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reference_identifier",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_reference_service_request",
    )


class ServiceRequest(TimeStampedModel):
    """service request model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="service_request_identifier", blank=True
    )
    # TODO: instantiates_canonical = models.ManyToManyField(
    #     "base.Canonical",
    #     related_name="service_request_instantiates_canonical",
    #     blank=True,
    # )
    instantiates_uri = models.URLField(null=True)
    based_on = models.ManyToManyField(
        ServiceRequestBasedOnReference,
        blank=True,
        related_name="service_request_based_on",
    )
    replaces = models.ManyToManyField(
        ServiceRequestReference,
        blank=True,
        related_name="service_request_replaces",
    )
    requisition = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requisition",
    )
    status = models.CharField(
        max_length=255,
        choices=ServiceRequestStatus.choices,
        default=ServiceRequestStatus.ACTIVE,
    )
    intent = models.CharField(
        max_length=255,
        choices=ServiceRequestIntent.choices,
        default=ServiceRequestIntent.ORDER,
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="service_request_category", blank=True
    )
    priority = models.CharField(
        max_length=255,
        choices=ServiceRequestPriority.choices,
        default=ServiceRequestPriority.ROUTINE,
    )
    do_not_perform = models.BooleanField(default=False)
    code = models.ForeignKey(
        "activitydefinitions.ActivityDefinitionPlanDefinitionReference",
        related_name="service_request_code",
        on_delete=models.SET_NULL,
        null=True,
    )
    order_detail = models.ManyToManyField(
        ServiceRequestOrderDetail,
        blank=True,
    )
    quantity_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_quantity_quantity",
    )
    quantity_ratio = models.ForeignKey(
        "base.Ratio",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_quantity_ratio",
    )
    quantity_range = models.ForeignKey(
        "base.Range",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_quantity_range",
    )
    subject = models.ForeignKey(
        ServiceRequestSubjectReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_subject",
    )
    focus = models.ManyToManyField(
        Reference, related_name="service_request_focus", blank=True
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_encounter",
    )
    occurrence_date_time = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        "base.Period",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_occurrence_period",
    )
    occurrence_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_occurrence_timing",
    )
    as_needed = models.BooleanField(default=False)
    as_needed_for = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="service_request_as_needed_for",
    )
    authored_on = models.DateTimeField(null=True)
    requester = models.ForeignKey(
        ServiceRequestRequesterReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_requester",
    )
    additional_recipient = models.ManyToManyField(
        ServiceRequestRequesterReference,
        blank=True,
        related_name="service_request_additional_recipient",
    )
    performer_type = models.ForeignKey(
        CodeableConcept, on_delete=models.SET_NULL, null=True
    )
    performer = models.ManyToManyField(
        ServiceRequestPerformerReference,
        blank=True,
        related_name="service_request_performer",
    )
    location = models.ManyToManyField(
        "locations.LocationCodeableReference",
        blank=True,
        related_name="service_request_location",
    )
    reason = models.ManyToManyField(
        ServiceRequestReasonCodeableReference,
        blank=True,
        related_name="service_request_reason",
    )
    insurance = models.ManyToManyField(
        "coverages.CoverageClaimResponseReference",
        blank=True,
        related_name="service_request_insurance",
    )
    supporting_info = models.ManyToManyField(
        "base.CodeableReference",
        related_name="service_request_supporting_info",
        blank=True,
    )
    specimen = models.ManyToManyField(
        "specimens.SpecimenReference",
        related_name="service_request_specimen",
        blank=True,
    )
    body_site = models.ManyToManyField(
        CodeableConcept, related_name="service_request_body_site", blank=True
    )
    body_structure = models.ForeignKey(
        "bodystructures.BodyStructureReference",
        null=True,
        on_delete=models.SET_NULL,
        related_name="service_request_body_structure",
    )
    note = models.ManyToManyField(
        "base.Annotation", related_name="service_request_note", blank=True
    )
    patient_instruction = models.ManyToManyField(
        ServiceRequestPatientInstruction,
        related_name="service_request_patient_instruction",
        blank=True,
    )
    relevant_history = models.ManyToManyField(
        "provenances.ProvenanceReference",
        related_name="service_request_relevant_history",
        blank=True,
    )


class ServiceRequestPlanDefinitionReference(BaseReference):
    """service request plan definition reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_plan_definition_identifier",
    )
    plan_definition = models.ForeignKey(
        "plandefinitions.PlanDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_plan_definition_plan_definition",
    )
    service_request = models.ForeignKey(
        ServiceRequest,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_plan_definition_service_request",
    )


class ServiceRequestPlanDefinitionReferenceCodeableReference(TimeStampedModel):
    """service request plan definition reference codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_plan_definition_reference_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        ServiceRequestPlanDefinitionReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="service_request_plan_definition_reference_codeable_reference_reference",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('conditions', '0004_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0003_initial'),
        ('healthcareservices', '0001_initial'),
        ('medicationrequests', '0002_initial'),
        ('observations', '0002_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationrequestbasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_based_on_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='medication_request_based_on', to='medicationrequests.medicationrequestbasedonreference'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_information_source_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_information_source_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_information_source_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_information_source_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_information_source_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationrequestinformationsourcereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='information_source',
            field=models.ManyToManyField(blank=True, related_name='medication_request_information_source', to='medicationrequests.medicationrequestinformationsourcereference'),
        ),
        migrations.AddField(
            model_name='medicationrequestinsurancereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_insurance_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='medication_request_insurance', to='medicationrequests.medicationrequestinsurancereference'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='healthcare_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_healthcare_service', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_performer_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationrequestperformerreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='performer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_performer', to='medicationrequests.medicationrequestperformerreference'),
        ),
        migrations.AddField(
            model_name='medicationrequestprovinancereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_provinance_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='medicationrequestreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='medicationrequestreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_codeble_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='medicationrequestreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_reference', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='prior_prescription',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_prior_prescription', to='medicationrequests.medicationrequestreference'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='devices.device'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationrequestrequesterreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='requester',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_requester', to='medicationrequests.medicationrequestrequesterreference'),
        ),
        migrations.AddField(
            model_name='medicationrequestsubstitution',
            name='allowed_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_substitution_allowed_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationrequestsubstitution',
            name='reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_substitution_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='substitution',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_substitution', to='medicationrequests.medicationrequestsubstitution'),
        ),
        migrations.AddField(
            model_name='medicatonrequestreasoncodealblereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_reason_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicatonrequestreasoncodealblereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_request_reason_codeable_reference_reference', to='medicationrequests.medicationrequestreasonreference'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='medication_request_reason', to='medicationrequests.medicatonrequestreasoncodealblereference'),
        ),
    ]



"""medication requests models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Period,
    Quantity,
    Reference,
    TimeStampedModel,
)
from dfhir.base.models import Quantity as Duration
from dfhir.devices.models import DeviceCodeableReference
from dfhir.encounters.models import EncounterReference
from dfhir.medicationrequests.choices import (
    MedicationIntent,
    MedicationRequestPriority,
    MedicationRequestStatus,
)
from dfhir.medications.models import MedicationCodeableReference
from dfhir.organizations.models import Organization
from dfhir.patients.models import Patient
from dfhir.patients.models import PatientGroupReference as SubjectReference
from dfhir.practitioners.models import (
    Practitioner,
    PractitionerPractitionerRoleReference,
)
from dfhir.provenances.models import ProvenanceReference


class MedicationRequestCategory(TimeStampedModel):
    """medication request category model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class MedicationRequestMedicationCode(TimeStampedModel):
    """medication request medication codes."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class MedicationRequestReferenceType(TimeStampedModel):
    """medication request reference type."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class MedicationRequestReason(TimeStampedModel):
    """medication requests reasons."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class AdditionalIllustration(TimeStampedModel):
    """additional illustration model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class ReferenceAsNeededFor(TimeStampedModel):
    """as needed for model."""

    code = models.CharField(max_length=255, null=False)
    display = models.CharField(max_length=255, null=True)


class DosageSite(TimeStampedModel):
    """dosage site model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class DosageRoute(TimeStampedModel):
    """dosage route model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class DosageMethod(TimeStampedModel):
    """dosage method model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class MedicationRequestBasedOnReference(BaseReference):
    """medication request based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_based_on_identifier",
    )
    medication_request = models.ForeignKey(
        "MedicationRequest",
        on_delete=models.DO_NOTHING,
        related_name="medication_request_based_on_reference",
        null=True,
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_based_on_care_plan",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_based_on_service_request",
    )
    immunization_recommendation = models.ForeignKey(
        "immunizationrecommendations.ImmunizationRecommendation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_based_on_immunization_recommendation",
    )
    # TODO: request_orchestration = models.ForeignKey(
    #     "RequestOrchestration", on_delete=models.DO_NOTHING, null=True
    # )


class MedicationRequestReference(BaseReference):
    """medication reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_reference_identifier",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        related_name="medication_request_reference",
        null=True,
    )


class medicationRequestInformationSourceReference(BaseReference):
    """medication request information source reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_information_source_reference_identifier",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_information_source_reference_patient",
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_information_source_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_information_source_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", on_delete=models.DO_NOTHING, null=True
    )
    organization = models.ForeignKey(
        Organization,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_information_source_reference_organization",
    )


class MedicationRequestRequesterReference(BaseReference):
    """medication request requester reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_identifier",
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        Organization,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_organization",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester_reference_related_person",
    )
    device = models.ForeignKey("devices.Device", on_delete=models.DO_NOTHING, null=True)


class MedicationRequestPerformerReference(BaseReference):
    """medication request performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_performer_reference_identifier",
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_performer_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        Organization,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_performer_reference_organization",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_performer_reference_patient",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_performer_reference_device_definition",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", null=True, on_delete=models.SET_NULL
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam", null=True, on_delete=models.SET_NULL
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_performer_reference_healthcare_service",
    )


class MedicationRequestReasonReference(BaseReference):
    """medication request  reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_codeble_reason_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition", null=True, on_delete=models.SET_NULL
    )
    observation = models.ForeignKey(
        "observations.Observation",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_reason_reference_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_reason_reference_diagnostic_report",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_reason_reference_procedure",
    )


class MedicatonRequestReasonCodealbleReference(TimeStampedModel):
    """medication request reason codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_reason_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        MedicationRequestReasonReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_reason_codeable_reference_reference",
    )


class DispenseRequestInitialFill(TimeStampedModel):
    """dispense request initial fill model."""

    quantity = models.ForeignKey(
        Quantity,
        null=True,
        on_delete=models.SET_NULL,
        related_name="dispense_request_initial_fill_quantity",
    )
    duration = models.ForeignKey(
        Duration,
        null=True,
        on_delete=models.SET_NULL,
        related_name="dispense_request_initial_fill_duration",
    )


class DispenseRequest(TimeStampedModel):
    """dispense requester model."""

    initial_fill = models.ForeignKey(
        DispenseRequestInitialFill,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_initial_fill",
    )
    dispense_interval = models.ForeignKey(
        Duration,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_dispense_interval",
    )
    validity_period = models.ForeignKey(
        Period,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_validity_period",
    )
    number_of_repeats_allowed = models.IntegerField(null=True)
    quantity = models.ForeignKey(
        Quantity,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_quantity",
    )
    expected_supply_duration = models.ForeignKey(
        Duration, on_delete=models.SET_NULL, null=True
    )
    dispenser = models.ForeignKey(
        OrganizationReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_dispenser",
    )
    dispense_instruction = models.ManyToManyField(
        Annotation, blank=True, related_name="dispense_request_instruction"
    )
    dose_administration_aid = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="dispense_request_dose_administration_aid",
    )


class MedicationRequestSubstitution(TimeStampedModel):
    """medication request substitution model."""

    allowed_boolean = models.BooleanField(default=True)
    allowed_codeable_concept = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.SET_NULL,
        related_name="medication_request_substitution_allowed_codeable_concept",
    )
    reason = models.ForeignKey(
        CodeableConcept,
        related_name="medication_request_substitution_reason",
        null=True,
        on_delete=models.SET_NULL,
    )


class MedicationRequestInsuranceReference(BaseReference):
    """medication request insurance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_insurance_reference_identifier",
    )
    coverage = models.ForeignKey(
        "coverages.Coverage",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_insurance_reference_coverage",
    )
    claim_response = models.ForeignKey(
        "claimresponses.ClaimResponse",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_insurance_reference_claim_response",
    )


class MedicationRequestProvinanceReference(BaseReference):
    """medication request insurance reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_provinance_reference_identifier",
    )
    provenance = models.ForeignKey(
        "provenances.Provenance",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_provinance_reference_provenance",
    )


class MedicationRequest(TimeStampedModel):
    """medication request model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="medication_request_identifier"
    )
    based_on = models.ManyToManyField(
        MedicationRequestBasedOnReference,
        blank=True,
        related_name="medication_request_based_on",
    )
    prior_prescription = models.ForeignKey(
        MedicationRequestReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_prior_prescription",
    )
    group_identifier = models.ForeignKey(
        Identifier,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_group_identifier",
    )
    status = models.CharField(
        max_length=255,
        null=True,
        choices=MedicationRequestStatus.choices,
    )
    status_reason = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_status_reason",
    )
    status_changed = models.DateTimeField(null=True)
    intent = models.CharField(
        max_length=255, null=True, choices=MedicationIntent.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="medication_request_category"
    )
    priority = models.CharField(
        max_length=255,
        null=True,
        choices=MedicationRequestPriority.choices,
    )
    do_not_perform = models.BooleanField(default=False)
    medication = models.ForeignKey(
        MedicationCodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_mepatientdication",
    )
    subject = models.ForeignKey(
        SubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_subject",
        null=True,
    )
    information_source = models.ManyToManyField(
        medicationRequestInformationSourceReference,
        blank=True,
        related_name="medication_request_information_source",
    )
    encounter = models.ForeignKey(
        EncounterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_encounter",
    )
    supporting_information = models.ManyToManyField(
        Reference, blank=True, related_name="medication_request_supporting_information"
    )
    authored_on = models.DateTimeField(null=True)
    requester = models.ForeignKey(
        MedicationRequestRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_requester",
    )
    reported = models.BooleanField(default=True)
    performer_type = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_performer_type",
    )
    performer = models.ForeignKey(
        MedicationRequestPerformerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_performer",
    )
    device = models.ManyToManyField(
        DeviceCodeableReference,
        blank=True,
        related_name="medication_request_device",
    )
    recorder = models.ForeignKey(
        PractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_recorder",
        null=True,
    )
    reason = models.ManyToManyField(
        MedicatonRequestReasonCodealbleReference,
        blank=True,
        related_name="medication_request_reason",
    )
    course_of_therapy_type = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_course_of_therapy_type",
    )
    insurance = models.ManyToManyField(
        MedicationRequestInsuranceReference,
        blank=True,
        related_name="medication_request_insurance",
    )
    note = models.ManyToManyField(
        Annotation, blank=True, related_name="medication_request_note"
    )
    rendered_dosage_instruction = models.TextField(null=True)
    effective_dose_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_request_effective_dose_period",
    )
    # TODO: dosage_instruction = models.ManyToManyField("dosages.Dosage", blank=True)
    dispense_request = models.ForeignKey(
        DispenseRequest,
        on_delete=models.DO_NOTHING,
        related_name="medication_request_dispense_request",
        null=True,
    )
    substitution = models.ForeignKey(
        MedicationRequestSubstitution,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_request_substitution",
    )
    event_history = models.ManyToManyField(
        ProvenanceReference,
        blank=True,
        related_name="medication_request_event_history",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubstanceIngredient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('quantity', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='substance_ingredient_quantity', to='base.ratio')),
                ('substance_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_ingredient_substance', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Substance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('expiry', models.DateTimeField(null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='substances_category', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='substances_identifier', to='base.identifier')),
                ('ingredient', models.ManyToManyField(blank=True, related_name='substances_ingredient', to='substances.substanceingredient')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SubstanceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_reference_identifier', to='base.identifier')),
                ('substance', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_reference_substance', to='substances.substance')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='substanceingredient',
            name='substance_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_ingredient_substance_reference', to='substances.substancereference'),
        ),
        migrations.CreateModel(
            name='SubstanceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='substance_codeable_reference_reference', to='substances.substancereference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Location choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class LocationForm(models.TextChoices):
    """Location form choices."""

    BUILDING = "building", _("Building")
    ROOM = "room", _("Room")
    WARD = "ward", _("Ward")
    BED = "bed", _("Bed")
    WING = "wing", _("Wing")
    CORRIDOR = "corridor", _("Corridor")
    VEHICLE = "vehicle", _("Vehicle")
    ROAD = "road", _("Road")
    VIRTUAL = "virtual", _("Virtual")


class LocationStatus(models.TextChoices):
    """Location status choices."""

    ACTIVE = "active", _("Active")
    SUSPENDED = "suspended", _("Suspended")
    INACTIVE = "inactive", _("Inactive")


class LocationMode(models.TextChoices):
    """Location mode choices."""

    INSTANCE = "instance", _("Instance")
    KIND = "kind", _("Kind")


class OperationalStatus(models.TextChoices):
    """Operational status choices."""

    CLOSED = "closed", _("Closed")
    OCCUPIED = "occupied", _("Occupied")
    UNOCCUPIED = "unoccupied", _("Unoccupied")
    ISOLATED = "isolated", _("Isolated")
    HOUSEKEEPING = "housekeeping", _("Housekeeping")
    CONTAMINATED = "contaminated", _("Contaminated")


class LocationCharacteristicChoices(models.TextChoices):
    """Location characteristic choices."""

    WHEEL_CHAIR = "wheel_chair", _("Wheel Chair Accessible")
    HAS_TRANSLATION_SERVICE = (
        "has_translation_service",
        _("Translation services available"),
    )
    HAS_OXY_NITROGEN = "has_oxy_nitrogen", _("Oxygen and Nitrogen available")
    HAS_NEG_PRESSURE = "has_neg_pressure", _("Negative pressure rooms available")
    HAS_ISO_WARD = "has_iso_ward", _("Isolation ward")
    HAS_ICU = "has_icu", _("Intensive Care Unit")



"""Allergy intolerances views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import AllergyIntolerance
from .serializers import AllergyIntoleranceSerializer


class AllergyIntoleranceListView(APIView):
    """Allergy intolerance list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: AllergyIntoleranceSerializer(many=True)})
    def get(self, request):
        """Get all allergy intolerances."""
        allergy_intolerances = AllergyIntolerance.objects.all()
        serializer = AllergyIntoleranceSerializer(allergy_intolerances, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=AllergyIntoleranceSerializer,
        responses={201: AllergyIntoleranceSerializer},
    )
    def post(self, request):
        """Create an allergy intolerance."""
        serializer = AllergyIntoleranceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class AllergyIntoleranceDetailView(APIView):
    """Allergy intolerance detail view."""

    def get_object(self, pk):
        """Get an allergy intolerance object."""
        try:
            return AllergyIntolerance.objects.get(pk=pk)
        except AllergyIntolerance.DoesNotExist as error:
            raise Http404 from error

    @extend_schema(responses={200: AllergyIntoleranceSerializer})
    def get(self, request, pk=None):
        """Get an allergy intolerance."""
        allergy_intolerance = self.get_object(pk)
        serializer = AllergyIntoleranceSerializer(allergy_intolerance)
        return Response(serializer.data)

    @extend_schema(
        request=AllergyIntoleranceSerializer,
        responses={200: AllergyIntoleranceSerializer},
    )
    def patch(self, request, pk):
        """Update an allergy intolerance."""
        allergy_intolerance = self.get_object(pk)
        serializer = AllergyIntoleranceSerializer(
            allergy_intolerance, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete an allergy intolerance."""
        allergy_intolerance = self.get_object(pk)
        allergy_intolerance.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Coverage choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class CoverageStatusChoices(models.TextChoices):
    """Coverage status choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class CoverageKindChoices(models.TextChoices):
    """Coverage kind choices."""

    INSURANCE = "insurance", _("Insurance")
    SELF_PAY = "self-pay", _("Self Pay")
    OTHER = "other", _("Other")
